 /************************************************************************/
 /*                                                                      */
 /*                Centre for Speech Technology Research                 */
 /*                     University of Edinburgh, UK                      */
 /*                       Copyright (c) 1996,1997                        */
 /*                        All Rights Reserved.                          */
 /*                                                                      */
 /*  Permission to use, copy, modify, distribute this software and its   */
 /*  documentation for research, educational and individual use only, is */
 /*  hereby granted without fee, subject to the following conditions:    */
 /*   1. The code must retain the above copyright notice, this list of   */
 /*      conditions and the following disclaimer.                        */
 /*   2. Any modifications must be clearly marked as such.               */
 /*   3. Original authors' names are not deleted.                        */
 /*  This software may not be used for commercial purposes without       */
 /*  specific prior written permission from the authors.                 */
 /*                                                                      */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
 /*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  */
 /*  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE    */
 /*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   */
 /*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  */
 /*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         */
 /*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      */
 /*  THIS SOFTWARE.                                                      */
 /*                                                                      */
 /************************************************************************/
 /*                 Author: Richard Caley (rjc@cstr.ed.ac.uk)            */
 /*                   Date: Wed Mar 19 1997                              */
 /************************************************************************/
 /*                                                                      */
 /* UnitIndex which constructs the name of the relevent diphone from     */
 /* the names of a pair of phonemes.                                     */
 /*                                                                      */
 /************************************************************************/

#ifndef __SIMPLEDIPHONEUNITINDEX_H__
#define __SIMPLEDIPHONEUNITINDEX_H__

#include "EST.h"
#include "EST_TVector.h"
#include "festival.h"
#include "Phone.h"
#include "UnitIndex.h"

class SimpleDiphoneUnitIndex : public UnitIndex {
public:
  typedef EST_TList<Phone *> PList;
  typedef KVL<Phone *, PList *> PhoneMap;

private:
  static const char s_magic[];
  static const int s_magic_size;

  PhoneMap before_map;
  PhoneMap after_map;

public:

  SimpleDiphoneUnitIndex(void);
  virtual ~SimpleDiphoneUnitIndex(void);
  
  static bool recognise(FILE *stream);
  static UnitIndex *create(void); 

  virtual EST_String subtype_name(void) const {return "SimpleDiphone";};

  virtual EST_read_status fill_from(FILE *stream, int binary=1);
  virtual EST_write_status write_to(FILE *stream, int binary=1);

  virtual int SimpleDiphoneUnitIndex::set_property(EST_String property, ValueType value);

  virtual void index(void);

  bool lookup(const EST_String p1, const EST_String p2, EST_TList<UnitName> &options) const;

};

#endif
